#!/usr/bin/env python
# -*- coding: utf-8 -*-

from collections import Counter
import codecs
import argparse
import re
import itertools

__author__ = "Rohit Gupta"

def pairwise(iterable):
        "s -> (s0,s1), (s1,s2), (s2, s3), ..."
        a, b = itertools.tee(iterable)
        next(b, None)
        return itertools.izip(a, b)


parser = argparse.ArgumentParser()
parser.add_argument('--corpus', dest='corpus_file', help='Language corpus to use. File <corpus>.txt is used as source of the corpus', metavar='corpus')
parser.add_argument('--terminals', dest='terminals_file', help='file containing syllable terminals (vowels) for the language (one per line)')
parser.add_argument('--topN', type=int, dest='topN', help='Number of syllables to print stats for')

args = parser.parse_args()

#corpus = "HindiCorpus" 
corpus = args.corpus_file
fname = corpus + ".txt"

with codecs.open(fname, encoding='utf-8') as f:
    word_count = Counter(f.read().split())


with codecs.open(fname, encoding='utf-8') as f:
    alphabet_count = Counter([x.strip() for x in f.read() if x.strip() != ''])
    #alphabet_count = Counter([x for x in f.read()])

###################################
with codecs.open(args.terminals_file, encoding='utf-8') as f:
    terminals = f.read().split()

terminals_regex = '([' + ''.join(terminals) + '])+'

with codecs.open(fname, encoding='utf-8') as f:
    clean_words = [x.strip() for x in f.read().split() if x.strip() != '']    
    vowels_list = [re.finditer('[aeiou]', word) for word in clean_words]
    syllables = list()
    for index, vowel_match in enumerate(vowels_list):
        vowel_match_edges, vowel_match_itr  = itertools.tee(vowel_match, 2)       
        vowel_match_list = (list(vowel_match_edges))
        if len(vowel_match_list) == 0:
            syllables.append(clean_words[index])
            continue
        else:
            syllables.append(clean_words[index][:vowel_match_list[0].end()])            
        if len(vowel_match_list) >= 2:
            for v1, v2 in pairwise(vowel_match_itr):
                syllables.append(clean_words[index][v1.end():v2.start()+1])        
        syllables.append(clean_words[index][vowel_match_list[-1].start()+1:])

    syllable_count = Counter(filter(None, syllables))

    

###################################

word_stats = corpus + "WordStats.txt"

with codecs.open(word_stats, encoding='utf-8', mode='w') as f:
    for word, count in word_count.most_common(args.topN):
        f.write(word)
        f.write('\t')
        f.write(str(count))
        f.write('\n')    

print 'WordStats written to: ' + word_stats

letter_stats = corpus + "LetterStats.txt"

with codecs.open(letter_stats, encoding='utf-8', mode='w') as f:
    for alphabet, count in alphabet_count.most_common(args.topN):
        f.write(alphabet)
        f.write('\t')
        f.write(str(count))
        f.write('\n')

print 'LetterStats written to: ' + letter_stats

syllable_stats = corpus + "SyllableStats.txt"

with codecs.open(syllable_stats, encoding='utf-8', mode='w') as f:
    for syllable, count in syllable_count.most_common(args.topN):
        f.write(syllable)
        f.write('\t')
        f.write(str(count))
        f.write('\n')

print 'SyllableStats written to: ' + syllable_stats